<?php
/*
 * Plugin Name: Per Product Flat Rate Shipping for WooCommerce
 * Version: 1.0.5
 * Description: Set seperate flat-rate shipping costs for both Domestic and International shipping on a per product basis in WooCommerce.
 * Author: Jeff Sherk
 * Author URI: https://wordpress.org/support/plugin/per-product-flat-rate-shipping-for-wc/
 * Plugin URI: https://wordpress.org/plugins/per-product-flat-rate-shipping-for-wc/
 * Donate link: https://www.paypal.me/jsherk/10usd
 * License: GPLv3
 * License URI: http://www.gnu.org/licenses/gpl-3.0
 */

//NOTE:
//  Do a search and replace for   jdsbtp_   and replace it with   jdssomethingunique_
//  Then set the plugin_admin_name and plugin_admin_link below.
$jdsppfrs_plugin_admin_name = "Per Product Flat Rate Shipping for WC";
$jdsppfrs_plugin_admin_link = "per-product-flat-rate-shipping-for-wc";
$jdsppfrs_plugin_admin_donate_url = "https://www.paypal.me/jsherk/10usd";
$jdsppfrs_plugin_admin_review_url = "https://wordpress.org/support/plugin/".$jdsppfrs_plugin_admin_link."/reviews/#new-post";
 
/* ****************************************************************************** */
function jdsppfrs_add_links_to_admin_plugins_page($links) {

	global $jdsppfrs_plugin_admin_name;
	global $jdsppfrs_plugin_admin_link;
	global $jdsppfrs_plugin_admin_donate_url;
	global $jdsppfrs_plugin_admin_review_url;

	$review_url = $jdsppfrs_plugin_admin_review_url;
	$review_url = esc_url($review_url);
	$review_link = '<a href="'.$review_url.'">Leave a REVIEW</a>';
	array_unshift( $links, $review_link );

	$donate_url = $jdsppfrs_plugin_admin_donate_url;
	$donate_url = esc_url($donate_url);
	$donate_link = '<a href="'.$donate_url.'">DONATE</a>'; //DONATE
	array_unshift( $links, $donate_link ); //DONATE

	//$url = get_admin_url() . 'options-general.php?page='.$jdsppfrs_plugin_admin_link;
	$url = get_admin_url() . 'admin.php?page=wc-settings&tab=shipping&section=jdsppfrs_flat_rate_shipping';
	
	$url = esc_url($url);
	$settings_link = '<a href="'.$url.'">Settings</a>';
	array_unshift( $links, $settings_link );

	return $links;
}
add_filter( 'plugin_action_links_'.plugin_basename(__FILE__), 'jdsppfrs_add_links_to_admin_plugins_page' );


/* ****************************************************************************** */
function jdsppfrs_add_meta_to_admin_plugins_page( $links, $file ) {

	global $jdsppfrs_plugin_admin_name;
	global $jdsppfrs_plugin_admin_link;
	global $jdsppfrs_plugin_admin_donate_url;
	global $jdsppfrs_plugin_admin_review_url;

	if ( strpos( $file, plugin_basename(__FILE__) ) !== false ) {

		$review_url = $jdsppfrs_plugin_admin_review_url;
		$review_url = esc_url($review_url);

		$donate_url = $jdsppfrs_plugin_admin_donate_url;
		$donate_url = esc_url($donate_url);

		//$url = get_admin_url() . 'options-general.php?page='.$jdsppfrs_plugin_admin_link;
		$url = get_admin_url() . 'admin.php?page=wc-settings&tab=shipping&section=jdsppfrs_flat_rate_shipping';
		$url = esc_url($url);

		$donate_button = plugins_url( 'dbut-small.png', __FILE__ );
		$review_stars = plugins_url( 'stars-small.png', __FILE__ );
		$new_links = array('<a href="'.$url.'">Settings</a>', '<a href="'.$review_url.'">Leave a REVIEW <img style="vertical-align:text-top;" height="12" src="'.$review_stars.'"></a>', '<a href="'.$donate_url.'">Thanks for supporting me! <img style="vertical-align:bottom;" height="20" src="'.$donate_button.'"></a>'); //REVIEW & DONATE

		$links = array_merge( $links, $new_links );

	}
	
	return $links;
}
add_filter( 'plugin_row_meta', 'jdsppfrs_add_meta_to_admin_plugins_page', 10, 2 );


/* ****************************************************************************** */
function jdsppfrs_add_admin_settings_menu() {
	global $jdsppfrs_plugin_admin_name;
	global $jdsppfrs_plugin_admin_link;
	add_options_page( $jdsppfrs_plugin_admin_name.' by Jeff Sherk', $jdsppfrs_plugin_admin_name, 'activate_plugins', $jdsppfrs_plugin_admin_link, 'jdsppfrs_show_plugin_options' );
	//Add additional option pages as necessary
	//add_options_page( $jdsppfrs_plugin_admin_name.' by Jeff Sherk', $jdsppfrs_plugin_admin_name.' Extras', 'activate_plugins', $jdsppfrs_plugin_admin_link.'-extra', 'jdsppfrs_show_plugin_options_extra' );
}
add_action( 'admin_menu', 'jdsppfrs_add_admin_settings_menu' );


/* ****************************************************************************** */
function jdsppfrs_hide_options_page() {
	global $jdsppfrs_plugin_admin_link;
	remove_submenu_page('options-general.php', $jdsppfrs_plugin_admin_link);
}
add_action('admin_menu', 'jdsppfrs_hide_options_page', 999);


/* ****************************************************************************** */
function jdsppfrs_shipping_method() {
	if ( ! class_exists( 'jdsppfrs_WC_Per_Product_Flat_Rate_Shipping' ) ) {
		class jdsppfrs_WC_Per_Product_Flat_Rate_Shipping extends WC_Shipping_Method {
			
			public function __construct() {
				$this->id                 = 'jdsppfrs_flat_rate_shipping'; 
				$this->method_title       = __( 'Per Product Flat-Rate Shipping');  
				$this->method_description = __( 'Choose individual product shipping prices for Domestic and International flat-rate shipping.<br><br><b>Specify the "Flat-rate Domestic Shipping Cost" and the "Flat-rate International Shipping Cost" in the Product Data Shipping section on each individual products edit page.</b>'); 
				$this->init();
				
				// Setup default settings for the form fields
				if ( ! isset($this->settings['enabled']) ) {
					$this->settings['enabled'] = 'yes';
				}
				if ( ! isset($this->settings['domestic_country']) ) {
					$this->settings['domestic_country'] = 'US';
				}
				if ( ! isset($this->settings['shipping_name_domestic']) ) {
					$this->settings['shipping_name_domestic'] = 'Domestic';
				}
				if ( ! isset($this->settings['shipping_name_international']) ) {
					$this->settings['shipping_name_international'] = 'International';
				}
			}
			
			function init() {
				// Load the settings API
				$this->init_form_fields(); 
				$this->init_settings(); 
				// Save settings in admin if you have any defined
				add_action( 'woocommerce_update_options_shipping_' . $this->id, array( $this, 'process_admin_options' ) );
			}
			
			function init_form_fields() {
				// Use settings in __contruct to set the default values for these
				$this->form_fields = array(
				 'enabled' => array(
						'title' => __( 'Enable'),
						'type' => 'checkbox',
						'description' => __( 'Enable this shipping method.'),
						'default' => $this->settings['enabled']
					),
				
				 'domestic_country' => array(
						'title' => __( 'Domestic Country'),
						'type' => 'text',
						'description' => __( 'Use <a href='.chr(34).'https://www.iban.com/country-codes'.chr(34).' target='.chr(34).'_blank'.chr(34).'>2-letter uppercase country code (Alpha-2 code)</a> to determine when to use Domestic Shipping.<br>Example: If you use US for United States, the US will use the Domestic shipping rate, and all other countries except US, will use International shipping rate.'),
						'default' => $this->settings['domestic_country']
					),
				 
				 'shipping_name_domestic' => array(
						'title' => __( 'Domestic Shipping Name'),
						'type' => 'text',
						'description' => __( 'This is what your customers will see as one of their shipping options, when shipping country matches the Domestic Country above.'),
						'default' => $this->settings['shipping_name_domestic']
					),
				 
				 'shipping_name_international' => array(
						'title' => __( 'International Shipping Name'),
						'type' => 'text',
						'description' => __( 'This is what your customers will see as one of their shipping options, when shipping country does NOT match the Domestic Country above.'),
						'default' => $this->settings['shipping_name_international']
					),
				
				);
			}
			
			public function calculate_shipping( $package = array() ) {
				
				//mail("","WC package contents", print_r($package, 1) ); //DEBUG only
				
				$enabled = $this->settings['enabled']; // Is this shipping method enabled ??
				if ($enabled == "yes") {
				
					$domestic_country = $this->settings['domestic_country'];
					$rate_country = $package["destination"]["country"];
					if ($rate_country == $domestic_country) {
						$rate_title = $this->settings['shipping_name_domestic']; // Domestic shipping
					} else {
						$rate_title = $this->settings['shipping_name_international']; // International shipping
					}
					
					$shipping_cost = 0;
					foreach ($package['contents'] as $key => $val) {
						// Create empty array to store each items details in
						$item = array();
						
						// Add info about each item that we might want to use to calculate the shipping costs. Currently we are only using SKU
						$data_array = (array)$val['data']; // Need to access a protected key: Object->data->data:protected  ... Convert object to array first. This converts "data:protected" to " * data"
						$item['sku'] = $data_array["\0*\0data"]['sku']; // Then need to use "\0*\0data" in double-quotes to access the protected key.
						$item['name'] = $data_array["\0*\0data"]['name'];
						$item['slug'] = $data_array["\0*\0data"]['slug'];
						$item['price'] = $data_array["\0*\0data"]['price'];
						$item['regular_price'] = $data_array["\0*\0data"]['regular_price'];
						$item['sale_price'] = $data_array["\0*\0data"]['sale_price'];
						$item['weight'] = $data_array["\0*\0data"]['weight'];
						$item['length'] = $data_array["\0*\0data"]['length'];
						$item['width'] = $data_array["\0*\0data"]['width'];
						$item['height'] = $data_array["\0*\0data"]['height'];
						$item['shipping_class_id'] = $data_array["\0*\0data"]['shipping_class_id'];
						$item['product_id'] = $val['product_id'];
						$item['qty'] = $val['quantity'];
						$item['virtual'] = $data_array["\0*\0data"]['virtual']; // This will also be 0 because it actually automatically skips virtual products
						
						// Get shipping costs from custom fields
						$product = wc_get_product( $item['product_id'] );
						// Domestic rate a number?
						$shipping_cost_domestic = $product->get_meta('_flatrate_shipping_cost_domestic');
						if ( ! is_numeric($shipping_cost_domestic) ) {
							$shipping_cost_domestic = 0;
						} else if ($shipping_cost_domestic < 0) {
							$shipping_cost_domestic = 0;
						}
						// International rate a number?
						$shipping_cost_international = $product->get_meta('_flatrate_shipping_cost_international');
						if ( ! is_numeric($shipping_cost_international) ) {
							$shipping_cost_international = 0;
						} else if ($shipping_cost_international < 0) {
							$shipping_cost_international = 0;
						}
						
						// Which rate to use?
						if ($rate_country == $domestic_country) {
							// Use Domestic USA rate
							$rate = $shipping_cost_domestic;
						} else {
							// Use International rate
							$rate = $shipping_cost_international;
						}
						
						// Add to shipping cost
						$shipping_cost = $shipping_cost + ( $item['qty'] * $rate );
					}
					
					if ($shipping_cost >= 0) { // Use >=0 and then the jdsppfrs_add_0_to_shipping_label function will show $0.00 as the amount.
						// Add the rate so it shows up as a shipping option
						$rate = array(
							'id' => $this->id,
							'label' => $rate_title,
							'cost' => $shipping_cost
						);
						$this->add_rate( $rate );
					}
				}
			}
		}
	}
}
add_action( 'woocommerce_shipping_init', 'jdsppfrs_shipping_method' );


/* ****************************************************************************** */
function jdsppfrs_add_0_to_shipping_label($label, $method) {
	// Default for $0 shipping rate is to not show any amount at all.
	// So if shipping rate is $0, then concatenate ": $0.00" to the label.
	if ( ! ( $method->cost > 0 ) ) {
		$label .= ': ' . wc_price(0);
	} 
	return $label;
}
add_filter( 'woocommerce_cart_shipping_method_full_label', 'jdsppfrs_add_0_to_shipping_label', 10, 2 );


/* ****************************************************************************** */
// Show field in Shipping tab of product data
function jdsppfrs_custom_shipping_field_domestic() {
	$args = array(
		'id' => '_flatrate_shipping_cost_domestic',
		'label' => __( 'Flat-rate Domestic Shipping Cost', 'jdsspbs' ),
		'class' => 'jdsspbs-custom-field',
		'desc_tip' => true,
		'description' => __( 'Enter flat rate shipping amount for domestic orders.', 'jdsspbs' ),
	);
	woocommerce_wp_text_input( $args );
}
add_action( 'woocommerce_product_options_shipping', 'jdsppfrs_custom_shipping_field_domestic' );


/* ****************************************************************************** */
// Show field in Shipping tab of product data
function jdsppfrs_custom_shipping_field_international() {
	$args = array(
		'id' => '_flatrate_shipping_cost_international',
		'label' => __( 'Flat-rate International Shipping Cost', 'jdsspbs' ),
		'class' => 'jdsspbs-custom-field',
		'desc_tip' => true,
		'description' => __( 'Enter flat rate shipping amount for international orders.', 'jdsspbs' ),
	);
	woocommerce_wp_text_input( $args );
}
add_action( 'woocommerce_product_options_shipping', 'jdsppfrs_custom_shipping_field_international' );


/* ****************************************************************************** */
// Save the data entered in into the custom fields on Shipping tab of product data
function jdsppfrs_save_custom_shipping_fields( $post_id ) {
	$product = wc_get_product( $post_id );
	// domestic shipping
	$title = isset( $_POST['_flatrate_shipping_cost_domestic'] ) ? sanitize_text_field($_POST['_flatrate_shipping_cost_domestic']) : '';
	$product->update_meta_data( '_flatrate_shipping_cost_domestic', $title);
	$product->save();
	// international shipping
	$title = isset( $_POST['_flatrate_shipping_cost_international'] ) ? sanitize_text_field($_POST['_flatrate_shipping_cost_international']) : '';
	$product->update_meta_data( '_flatrate_shipping_cost_international', $title);
	$product->save();
}
add_action( 'woocommerce_process_product_meta', 'jdsppfrs_save_custom_shipping_fields' );


/* ****************************************************************************** */
function jdsppfrs_add_shipping_method( $methods ) {
	$methods['jdsppfrs_shipping_price_by_product'] = 'jdsppfrs_WC_Per_Product_Flat_Rate_Shipping';
	return $methods;
}
add_filter( 'woocommerce_shipping_methods', 'jdsppfrs_add_shipping_method' );


/* ****************************************************************************** */
// Validate the domestic country field and make sure it is only two upper case letters
function jdsppfrs_validate_domestic_country($option_name, $option_value) {
	$my_options = get_option('woocommerce_jdsppfrs_flat_rate_shipping_settings');
	$domestic_country = $my_options['domestic_country'];
	$domestic_country = strtoupper(trim($domestic_country));
	if (strlen($domestic_country) < 2) {
		$domestic_country = "";
	} else if (strlen($domestic_country) > 2) {
		$domestic_country = substr($domestic_country, 0, 2);
	}
	$my_options['domestic_country'] = $domestic_country;
	update_option( 'woocommerce_jdsppfrs_flat_rate_shipping_settings', $my_options, true );
}
add_action('update_option_woocommerce_jdsppfrs_flat_rate_shipping_settings', 'jdsppfrs_validate_domestic_country', 10, 2);
add_action('add_option_woocommerce_jdsppfrs_flat_rate_shipping_settings', 'jdsppfrs_validate_domestic_country', 10, 2);


/* ****************************************************************************** */
function jdsppfrs_show_plugin_options() {

	global $jdsppfrs_plugin_admin_name;
	global $jdsppfrs_plugin_admin_link;
	global $jdsppfrs_plugin_admin_donate_url;
	global $jdsppfrs_plugin_admin_review_url;

	$donate_url = $jdsppfrs_plugin_admin_donate_url;
	$review_url = $jdsppfrs_plugin_admin_review_url;

	include_once( ABSPATH . 'wp-admin/includes/plugin.php' ); //Required for Front End users but not for Admin area
	/* if (is_plugin_inactive("woocommerce/woocommerce.php")) { //check if specific plugin is active

		?>
		<div class="wrap">
			<h1><?php _e( $jdsppfrs_plugin_admin_name ); ?></h1>
			<h3><i style='color:red;'>NOTICE: WooCommerce must be installed and activated to use this plugin.</i></h3>
		</div>
		<?php

	} else { */

	?>
	<div class="wrap">
		<h1><?php _e( $jdsppfrs_plugin_admin_name ); ?></h1>
	</div>
	<?php

	//CHECK IF OPTIONS WERE EVER SET OR NOT
	$update_options_firstrun = false;
	$my_options = get_option('jdsppfrs_options');

	$this_option = "option_checkbox";
	if ( !is_array($my_options) || !array_key_exists($this_option, $my_options) ) {
		$my_options[$this_option] = "off"; //Set default CHECKBOX to either on or off
		$update_options_firstrun = true;
	}

	$this_option = "option_textbox";
	if ( !is_array($my_options) || !array_key_exists($this_option, $my_options) ) {
		$my_options[$this_option] = ""; //Set default TEXTBOX to anything
		$update_options_firstrun = true;
	}

	$this_option = "option_select";
	if ( !is_array($my_options) || !array_key_exists($this_option, $my_options) ) {
		$my_options[$this_option] = "0"; //Set default SELECT to anything but works best with intergers starting at 0
		$update_options_firstrun = true;
	}

	//Create the options on firstrun if they did not exist
	if ($update_options_firstrun) {
		update_option( 'jdsppfrs_options', $my_options, true );
		$my_options = get_option('jdsppfrs_options'); //Make sure we have most up to date copy of options
	}

	//SAVE NEW SETTINGS
	$settings_saved = false;
	if ( isset( $_POST[ 'buttonSaveChanges' ] ) && current_user_can('activate_plugins') && check_admin_referer('save-changes-to-form') ) { //Check if nonce is valid using check_admin_referer or wp_verify_nonce

		$this_option = "option_checkbox";
		$post_data[$this_option] = sanitize_text_field($_POST[$this_option]);

		$this_option = "option_textbox";
		$post_data[$this_option] = sanitize_text_field($_POST[$this_option]);

		$this_option = "option_select";
		$post_data[$this_option] = sanitize_text_field($_POST[$this_option]);

		update_option( 'jdsppfrs_options', $post_data, true );
		$my_options = get_option('jdsppfrs_options'); //Make sure we have most up to date copy of options

		$settings_saved = true;

	}
	?>

	<script>
		jQuery(document).ready(function($){
			$('#buttonSaveChanges').click(function(){ $(this).prop('value', '.....  SAVING  CHANGES  .....'); }); //Change Save Changes button text and disable button when clicked
		});
	</script>
	<?php if ( $settings_saved ) : ?>
	<script>
		jQuery(document).ready(function($){
			$('.fadeoutChangesSaved').click(function(){ $(this).fadeOut('fast'); }); //fadeout Changes Saved message if clicked
			setTimeout(function(){ $('.fadeoutChangesSaved').fadeOut("slow"); } ,2000); //fadeout Changes Saved message after 5 seconds
		});
	</script>
	<div id="message" class="updated fadeoutChangesSaved">
		<p><strong><?php _e( 'Changes saved.' ) ?></strong></p>
	</div>
			
	<?php endif ?>

	<form method="post" action="">

		<p>
			<?php
			$this_option_name = "option_checkbox";
			$checked = "";
			if ($my_options[$this_option_name] == "on") {
				$checked = "CHECKED";
			}
			?>
			<input type="hidden" name="<?php echo $this_option_name; ?>" value="off" >
			<input type="checkbox" id="<?php echo $this_option_name; ?>" name="<?php echo $this_option_name; ?>" <?php echo $checked; ?> value="on" >
			<label for="<?php echo $this_option_name; ?>"><?php _e( 'Check to ENABLE this option.' ) ?></label>
			<br><span style="font-size: 85%; font-style: italic;">NOTE: This is a note.</span>
		</p>

		<p>
			<?php
			$this_option_name = "option_textbox";
			$this_option_value = $my_options[$this_option_name];
			?>
			<label for="<?php echo $this_option_name; ?>"><?php _e( 'Enter text:' ) ?></label>
			<input type="textbox" id="<?php echo $this_option_name; ?>" name="<?php echo $this_option_name; ?>" value="<?php echo $this_option_value; ?>" size="50">
			<br><span style="font-size: 85%; font-style: italic;">NOTE: This is a note.</span>
		</p>

		<p>
			<?php
			$this_option_name = "option_select";
			$this_option_value = $my_options[$this_option_name];
			?>
			<label for="<?php echo $this_option_name; ?>"><?php _e( 'Select:' ) ?></label>
			<select id="<?php echo $this_option_name; ?>" name="<?php echo $this_option_name; ?>">
				<?php
					//START LOOP to populate option values and set $selected="SELECTED" for saved option. Defaults to first option in list if none selected.
					for ($i = 0; $i <= 3; $i++) {
						if ($this_option_value == $i) {
							$selected[$i] = "SELECTED";
						} else {
							$selected[$i] = "";
						}
					}
					?>
						<option value="0" <?php echo $selected[0]; ?> >0 - No selection</option>
						<option value="1" <?php echo $selected[1]; ?> >1 - One</option>
						<option value="2" <?php echo $selected[2]; ?> >2 - Two</option>
						<option value="3" <?php echo $selected[3]; ?> >3 - Three</option>
			</select>
			<br><span style="font-size: 85%; font-style: italic;">NOTE: This is a note.</span>
		</p>


<!-- ------------------------------------------------------------------------- -->
		<?php $pluginbut = plugins_url( 'dbut.png', __FILE__ ); $review_stars = plugins_url( 'stars-small.png', __FILE__ ); ?>
		<br><hr>How much is this plugin worth to you? A suggested <a href="<?php echo $donate_url; ?>">donation of $10</a> will help motivate me to keep this plugin updated!<br><a href="<?php echo $donate_url; ?>"><img width="175" src="<?php echo $pluginbut; ?>"></a>
		<br>Reviews are also very hepful. Please consider leaving a <a href="<?php echo $review_url; ?>">postive REVIEW by cllicking here <img style="vertical-align:text-top;" height="12" src="<?php echo $review_stars; ?>"></a><hr>

		<?php
			wp_nonce_field('save-changes-to-form'); //Add nonce hidden fields
		?>
		<p class="submit">
			<input class="button-primary buttonSaveChanges" id="buttonSaveChanges" name="buttonSaveChanges" type="submit" value="<?php _e( 'Save Changes' ) ?>" />
		</p>

	</form>

		<?php
    /* } //check if specific plugin is active*/

}
?>